@@ -37,7 +37,7 @@
 #' @references
 #' \cite{A Petersen and HG Müller (2019). "Fréchet regression for random objects with Euclidean predictors." An. Stat. 47, 691-719.}
 #' \cite{Z Lin (2019). " Riemannian Geometry of Symmetric Positive Definite Matrices via Cholesky Decomposition." Siam. J. Matrix. Anal, A. 40, 1353–1370.}
-
+#' @importFrom Matrix forceSymmetric
 
 LFRCovCholesky <- function(x, M, xout, optns=list()){
   if(!is.matrix(x)&!is.vector(x)){
@@ -102,16 +102,19 @@ LFRCovCholesky <- function(x, M, xout, optns=list()){
     stop("M must be provided")
   }
   
-  if(class(M) == 'array'){
+  if(is.array(M)){
+    if (length(dim(M))!=3) {
+      stop('M must be an array or a list')
+    }
     MM = list()
-    if(class(M) == 'array'){
+    if(is.array(M)){
       for (i in 1:dim(M)[3]) {
         MM[[i]] = M[,,i]
       }
     }
     M = lapply(MM, function(X) (X+t(X))/2)
   }else{
-    if(!class(M)=="list"){
+    if(!is.list(M)){
       stop('M must be an array or a list')
     }
     M = lapply(M, function(X) (X+t(X))/2)
@@ -138,9 +141,6 @@ LFRCovCholesky <- function(x, M, xout, optns=list()){
       sL[i] =aux[i]*(1-t(mu1)%*%solve(mu2)%*%(x[i,]-x0))
     }
     s = sum(sL)
-    if(s == 0){
-      stop('Bandwidth too small')
-    }
     
     Mout = list()
     MM = M[idx]
@@ -191,52 +191,77 @@ LFRCovCholesky <- function(x, M, xout, optns=list()){
     if(p==1){
       bw_choice=SetBwRange(as.vector(x), as.vector(xout), kernel)
       objF=matrix(0,nrow=20,ncol=1)
-      aux1=as.matrix(seq(bw_choice$min,bw_choice$max,length.out=20))
-      for(i in 1:20){
-        for(j in 1:dim(x)[1]){
-          distj = distance(computeLFRSPD(setdiff(1:dim(x)[1],j),x[j],aux1[i]), M[[j]])
-          objF[i]=objF[i] + distj
-        }
+      aux1=as.matrix(seq(bw_choice$min,bw_choice$max,length.out=nrow(objF)))
+      for(i in 1:nrow(objF)){
+        #Try-catch statement in case bandwidth is too small and produces numerical issues
+        objF[i] = tryCatch({
+          sum(sapply(1:dim(x)[1],function(j){
+            distance(computeLFRSPD(setdiff(1:dim(x)[1],j),x[j],aux1[i]), M[[j]])
+          }))
+        }, error = function(e) {
+          return(NA)
+        })
+      }
+      if(sum(is.na(objF))==dim(objF)[1]*dim(objF)[2]){
+        stop("Bandwidth too small in cross-validation search")
+      }else{
+        ind=which(objF==min(objF,na.rm=TRUE))[1]
+        bwCV=aux1[ind]
       }
-      ind=which(objF==min(objF))[1]
-      bwCV=aux1[ind]
     }
     if(p==2){
       bw_choice1=SetBwRange(as.vector(x[,1]), as.vector(xout[,1]), kernel)
       bw_choice2=SetBwRange(as.vector(x[,2]), as.vector(xout[,2]), kernel)
       if(n<=30){
         objF=matrix(0,nrow=6,ncol=6)
-        aux1=seq(bw_choice1$min,bw_choice1$max,length.out=6)
-        aux2=seq(bw_choice2$min,bw_choice2$max,length.out=6)
-        for(i1 in 1:6){
-          for(i2 in 1:6){
-            for(j in 1:dim(x)[1]){
-              distj=distance(computeLFRSPD(setdiff(1:dim(x)[1],j),x[j,],c(aux1[i1],aux2[i2])), M[[j]])
-              objF[i1,i2]=objF[i1,i2]+distj
-            }
+        aux1=seq(bw_choice1$min,bw_choice1$max,length.out=nrow(objF))
+        aux2=seq(bw_choice2$min,bw_choice2$max,length.out=ncol(objF))
+        for(i1 in 1:nrow(objF)){
+          for(i2 in 1:ncol(objF)){
+            #Try-catch statement in case bandwidth is too small and produces numerical issues
+            objF[i1,i2] = tryCatch({
+              sum(sapply(1:dim(x)[1],function(j){
+                distance(computeLFRSPD(setdiff(1:dim(x)[1],j),x[j,],c(aux1[i1],aux2[i2])), M[[j]])
+              }))
+            }, error = function(e) {
+              return(NA)
+            })
           }
         }
-        ind=which(objF==min(objF),arr.ind = TRUE)
-        bwCV=c(aux1[ind[1]],aux2[ind[2]])
+        if(sum(is.na(objF))==dim(objF)[1]*dim(objF)[2]){
+          stop("Bandwidth too small in cross-validation search")
+        }else{
+          ind=which(objF==min(objF,na.rm=TRUE),arr.ind = TRUE)
+          bwCV=c(aux1[ind[1]],aux2[ind[2]])
+        }
       }else{
         randIndices=sample(dim(x)[1])
         groupIndices=cut(seq(1,dim(x)[1]),breaks=10,labels=FALSE)
-        cv10fold_compute=function(v){
+        cv10fold_compute=function(v,leaveIn){
           distance(computeLFRSPD(leaveIn,x[v,],c(aux1[i1],aux2[i2])),M[[v]])
         }
         objF=matrix(0,nrow=6,ncol=6)
-        aux1=seq(bw_choice1$min,bw_choice1$max,length.out=6)
-        aux2=seq(bw_choice2$min,bw_choice2$max,length.out=6)
-        for(i1 in 1:6){
-          for(i2 in 1:6){
-            for(j in 1:10){
-              leaveIn=setdiff(1:(dim(x)[1]),randIndices[groupIndices==j])
-              objF[i1,i2]=objF[i1,i2]+sum(sapply(randIndices[groupIndices==j],cv10fold_compute))
-            }
+        aux1=seq(bw_choice1$min,bw_choice1$max,length.out=nrow(objF))
+        aux2=seq(bw_choice2$min,bw_choice2$max,length.out=ncol(objF))
+        for(i1 in 1:nrow(objF)){
+          for(i2 in 1:ncol(objF)){
+            #Try-catch statement in case bandwidth is too small and produces numerical issues
+            objF[i1,i2] = tryCatch({
+              sum(sapply(1:10,function(j){
+                leaveIn=setdiff(1:(dim(x)[1]),randIndices[groupIndices==j])
+                sum(sapply(randIndices[groupIndices==j],function(v){cv10fold_compute(v,leaveIn)}))
+              }))
+            }, error = function(e) {
+              return(NA)
+            })
           }
         }
-        ind=which(objF==min(objF),arr.ind = TRUE)
-        bwCV=c(aux1[ind[1]],aux2[ind[2]])
+        if(sum(is.na(objF))==dim(objF)[1]*dim(objF)[2]){
+          stop("Bandwidth too small in cross-validation search")
+        }else{
+          ind=which(objF==min(objF,na.rm=TRUE),arr.ind = TRUE)
+          bwCV=c(aux1[ind[1]],aux2[ind[2]])
+        }
       }
     }
     bwCov=bwCV
