@@ -49,6 +49,7 @@
 #' @references
 #' \cite{Petersen, A. and Müller, H.-G. (2019). Fréchet regression for random objects with Euclidean predictors. The Annals of Statistics, 47(2), 691--719.}
 #' \cite{Petersen, A., Deoni, S. and Müller, H.-G. (2019). Fréchet estimation of time-varying covariance matrices from sparse data, with application to the regional co-evolution of myelination in the developing brain. The Annals of Applied Statistics, 13(1), 393--419.}
+#' @importFrom Matrix nearPD forceSymmetric
 
 LFRCovPower= function(x,y=NULL,M=NULL, xout,optns = list()){
   if(is.null(optns$corrOut)){
@@ -138,9 +139,6 @@ LFRCovPower= function(x,y=NULL,M=NULL, xout,optns = list()){
       sL[i] =aux[i]*(1-t(mu1)%*%solve(mu2)%*%(x[i,]-x0))
     }
     s = sum(sL)
-    if(s == 0){
-      stop('Bandwidth is too small')
-    }
 
     M_hat=array(0,c(dim(M)[1],dim(M)[1],1))
     if(alpha>0){
@@ -188,9 +186,6 @@ LFRCovPower= function(x,y=NULL,M=NULL, xout,optns = list()){
     for(i in 1:n){
       M[,,i] = (y[i,] - cmh[i,]) %*% t(y[i,] - cmh[i,])
     }
-    if(is.na(sum(bw))){
-      bw = bw1
-    }
   } else{
     if(is.null(M)){
       stop("y or M must be provided")
@@ -200,68 +195,97 @@ LFRCovPower= function(x,y=NULL,M=NULL, xout,optns = list()){
     } else{
       if(!is.array(M)){
         stop('M must be an array or a list')
+      } else if (length(dim(M))!=3) {
+        stop('M must be an array or a list')
       }
     }
     if(nrow(x)!=dim(M)[3]){
       stop("The number of rows of x must be the same as the number of covariance matrices in M")
     }
 
-    #CV for bw selection
-    if(is.na(sum(bw))){
-      if(p==1){
-        bw_choice=SetBwRange(as.vector(x), as.vector(xout), kernel)
-        objF=matrix(0,nrow=20,ncol=1)
-        aux1=as.matrix(seq(bw_choice$min,bw_choice$max,length.out=20))
-        for(i in 1:20){
-          for(j in 1:dim(x)[1]){
+    n=dim(M)[3]
+  }
+  #CV for bandwidth bw selection
+  if(is.na(sum(bw))){
+    if(p==1){
+      bw_choice=SetBwRange(as.vector(x), as.vector(xout), kernel)
+      objF=matrix(0,nrow=20,ncol=1)
+      aux1=as.matrix(seq(bw_choice$min,bw_choice$max,length.out=nrow(objF)))
+      for(i in 1:length(aux1)){
+        #Try-catch statement in case bandwidth is too small and produces numerical issues
+        objF[i] = tryCatch({
+          sum(sapply(1:dim(x)[1],function(j){
             aux=computeLFR_originalSpace(setdiff(1:dim(x)[1],j),x[j],aux1[i])-M[,,j]
-            objF[i]=objF[i]+sum(diag(aux%*%t(aux)))
-          }
-        }
-        ind=which(objF==min(objF))[1]
-        bwCV=aux1[ind]
-      }
-      if(p==2){
-        bw_choice1=SetBwRange(as.vector(x[,1]), as.vector(xout[,1]), kernel)
-        bw_choice2=SetBwRange(as.vector(x[,2]), as.vector(xout[,2]), kernel)
-        if(n<=30){
-          objF=matrix(0,nrow=6,ncol=6)
-          aux1=seq(bw_choice1$min,bw_choice1$max,length.out=6)
-          aux2=seq(bw_choice2$min,bw_choice2$max,length.out=6)
-          for(i1 in 1:6){
-            for(i2 in 1:6){
-              for(j in 1:dim(x)[1]){
-                aux=computeLFR_originalSpace(setdiff(1:dim(x)[1],j),x[j,],c(aux1[i1],aux2[i2]))-M[,,j]
-                objF[i1,i2]=objF[i1,i2]+sum(diag(aux%*%t(aux)))
-              }
-            }
-          }
-          ind=which(objF==min(objF),arr.ind = TRUE)
-          bwCV=c(aux1[ind[1]],aux2[ind[2]])
-        } else{
-          randIndices=sample(dim(x)[1])
-          groupIndices=cut(seq(1,dim(x)[1]),breaks=10,labels=FALSE)
-          cv10fold_compute=function(v){
-            aux=computeLFR_originalSpace(leaveIn,x[v,],c(aux1[i1],aux2[i2]))-M[,,v]
             sum(diag(aux%*%t(aux)))
+          }))
+        }, error = function(e) {
+          return(NA)
+        })
+      }
+      if(sum(is.na(objF))==dim(objF)[1]){
+        stop("Bandwidth too small in cross-validation search")
+      }
+      ind=which(objF==min(objF,na.rm=TRUE))[1]
+      bwCV=aux1[ind]
+    }
+    if(p==2){
+      bw_choice1=SetBwRange(as.vector(x[,1]), as.vector(xout[,1]), kernel)
+      bw_choice2=SetBwRange(as.vector(x[,2]), as.vector(xout[,2]), kernel)
+      if(n<=30){
+        objF=matrix(0,nrow=6,ncol=6)
+        aux1=seq(bw_choice1$min,bw_choice1$max,length.out=6)
+        aux2=seq(bw_choice2$min,bw_choice2$max,length.out=6)
+        for(i1 in 1:nrow(objF)){
+          for(i2 in 1:ncol(objF)){
+            #Try-catch statement in case bandwidth is too small and produces numerical issues
+            objF[i1,i2] = tryCatch({
+              sum(sapply(1:dim(x)[1],function(j){
+                aux=computeLFR_originalSpace(setdiff(1:dim(x)[1],j),x[j,],c(aux1[i1],aux2[i2]))-M[,,j]
+                sum(diag(aux%*%t(aux)))
+              }))
+            }, error = function(e) {
+              return(NA)
+            })
           }
-          objF=matrix(0,nrow=6,ncol=6)
-          aux1=seq(bw_choice1$min,bw_choice1$max,length.out=6)
-          aux2=seq(bw_choice2$min,bw_choice2$max,length.out=6)
-          for(i1 in 1:6){
-            for(i2 in 1:6){
-              for(j in 1:10){
+        }
+        if(sum(is.na(objF))==dim(objF)[1]*dim(objF)[2]){
+          stop("Bandwidth too small in cross-validation search")
+        }else{
+          ind=which(objF==min(objF,na.rm=TRUE),arr.ind = TRUE)
+          bwCV=c(aux1[ind[1]],aux2[ind[2]])
+        }
+      } else{
+        randIndices=sample(dim(x)[1])
+        groupIndices=cut(seq(1,dim(x)[1]),breaks=10,labels=FALSE)
+        cv10fold_compute=function(v,leaveIn){
+          aux=computeLFR_originalSpace(leaveIn,x[v,],c(aux1[i1],aux2[i2]))-M[,,v]
+          sum(diag(aux%*%t(aux)))
+        }
+        objF=matrix(0,nrow=6,ncol=6)
+        aux1=seq(bw_choice1$min,bw_choice1$max,length.out=6)
+        aux2=seq(bw_choice2$min,bw_choice2$max,length.out=6)
+        for(i1 in 1:nrow(objF)){
+          for(i2 in 1:ncol(objF)){
+            #Try-catch statement in case bandwidth is too small and produces numerical issues
+            objF[i1,i2] = tryCatch({
+              sum(sapply(1:10,function(j){
                 leaveIn=setdiff(1:(dim(x)[1]),randIndices[groupIndices==j])
-                objF[i1,i2]=objF[i1,i2]+sum(sapply(randIndices[groupIndices==j],cv10fold_compute))
-              }
-            }
+                sum(sapply(randIndices[groupIndices==j],function(v){cv10fold_compute(v,leaveIn)}))
+              }))
+            }, error = function(e) {
+              return(NA)
+            })
           }
-          ind=which(objF==min(objF),arr.ind = TRUE)
+        }
+        if(sum(is.na(objF))==dim(objF)[1]*dim(objF)[2]){
+          stop("Bandwidth too small in cross-validation search")
+        }else{
+          ind=which(objF==min(objF,na.rm=TRUE),arr.ind = TRUE)
           bwCV=c(aux1[ind[1]],aux2[ind[2]])
         }
       }
-      bw=bwCV
     }
+    bw=bwCV
   }
   Mout = list()
   if(corrOut){
