@@ -37,6 +37,7 @@
 #' \item{optns}{A list of control options used.}
 #'
 #' @examples
+#' \donttest{
 #' xin = seq(0,1,0.05)
 #' yin = lapply(xin, function(x) {
 #'   rnorm(100, rnorm(1,x + x^2,0.005), 0.05)
@@ -45,7 +46,6 @@
 #' xout = seq(0,1,0.1)
 #' res1 <- LocDenReg(xin=xin, yin=yin, xout=xout, optns = list(bwReg = 0.12, qSup = qSup))
 #' plot(res1)
-#'\donttest{
 #' xout <- xin
 #' hin = lapply(yin, function(y) hist(y, breaks = 50))
 #' res2 <- LocDenReg(xin=xin, hin=hin, xout=xout, optns = list(qSup = qSup))
@@ -54,6 +54,8 @@
 #' @references
 #' \cite{Petersen, A., & Müller, H.-G. (2019). "Fréchet regression for random objects with Euclidean predictors." The Annals of Statistics, 47(2), 691--719.}
 #' @export
+#' @importFrom fdadensity dens2quantile
+#' @importFrom pracma trapz
 
 LocDenReg <- function(xin=NULL, yin=NULL, hin=NULL, qin=NULL, xout=NULL, optns=list()) {
   
@@ -264,7 +266,7 @@ LocDenReg <- function(xin=NULL, yin=NULL, hin=NULL, qin=NULL, xout=NULL, optns=l
     }
   }
 
-  if (is.null(optnsReg$bw)) {
+  if (!"bw" %in% names(optnsReg)) {
     optnsReg$bw <- "CV"
   }
   if (!is.numeric(optnsReg$bw)) {
@@ -320,9 +322,9 @@ LocDenReg <- function(xin=NULL, yin=NULL, hin=NULL, qin=NULL, xout=NULL, optns=l
 
 # set up bandwidth range
 SetBwRange <- function(xin, xout, kernel_type) {
-  xinSt <- sort(xin)
+  xinSt <- unique(sort(xin))
   bw.min <- max(diff(xinSt), xinSt[2] - min(xout), max(xout) - xinSt[length(xin)-1])*1.1 / (ifelse(kernel_type == "gauss", 3, 1) * ifelse(kernel_type == "gausvar", 2.5, 1))
-  bw.max <- diff(range(xin))/3 / (ifelse(kernel_type == "gauss", 3, 1) * ifelse(kernel_type == "gausvar", 2.5, 1))
+  bw.max <- diff(range(xin))/3 #/ (ifelse(kernel_type == "gauss", 3, 1) * ifelse(kernel_type == "gausvar", 2.5, 1))
   if (bw.max < bw.min) {
     if (bw.min > bw.max*3/2) {
       #warning("Data is too sparse.")
@@ -365,7 +367,7 @@ bwCV <- function(xin, qin, xout, optns) {
 
     qfit <- lapply(seq_len(numFolds), function(foldidx) {
       testidx <- which(folds == foldidx)
-      res <- LocWassReg(xin = xin[-testidx,], qin = qin[-testidx,], xout = xin[testidx,],
+      res <- LocWassReg(xin = matrix(xin[-testidx,],ncol=p,byrow=TRUE), qin = matrix(qin[-testidx,],ncol=ncol(qin),byrow=TRUE), xout = matrix(xin[testidx,],ncol=p,byrow=TRUE),
                         optns = optns1)
       res # each row is a qt function
     })
@@ -421,7 +423,7 @@ bwCV <- function(xin, qin, xout, optns) {
   if(p==1){
     res <- optimize(f = objFctn, interval = bwRange[,1])$minimum
   }else{
-    res <- optim(par=rowMeans(bwRange),fn=objFctn,lower=bwRange[,1],upper=bwRange[,2],method='L-BFGS-B')$par
+    res <- optim(par=colMeans(bwRange),fn=objFctn,lower=bwRange[1,],upper=bwRange[2,],method='L-BFGS-B')$par
   }
   res
 }
